\name{em_model0}
\alias{em_model0}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculating MLE of Alternative Isoform Abundance
}
\description{
This function calculate the maximum likelihood estimation (MLE) for the alternative isoform abundance of a specific gene through the expectation-maximization (EM) algorithm.  This function can either be called by the user to calculate the MLE for isoform abundance of a gene in one sample or be called in the "isoform_estimation" function in this "rdiff" package to calculate the MLE for isoform abundance of a gene under model 0. In the latter, model 0 is defined as there's no differential expression of the gene and its alternative isoforms among all the samples/conditions. For detailed descriptions of how the models are difined, how the isoform abundance be quantified and how the algorithm works, please see the references.
}
\usage{
em_model0(A_matrix, N_vector, W_vector)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{A_matrix}{
%%     ~~Describe \code{A_matrix} here~~
The collapsed sampling rate matrix for a gene in one sample. This matrix is generated in one of the rseq output files with the extension ".sampling_rates" and is automatically read into a matrix in R through calling the function "read_file" of this "rdiff" package. See the references for a detailed description of sampling rate matrix.
}
  \item{N_vector}{
%%     ~~Describe \code{N_vector} here~~
The collapsed read counts vector for a gene in one sample. This vector is generated in one of the rseq output files with the extension ".sampling_rates" and is automatically read into a vector in R through calling the function "read_file" in this "rdiff" package. See the references for a detailed description of read counts vector.
}
  \item{W_vector}{
%%     ~~Describe \code{W_vector} here~~
The sampling-rate row-sum vector for a gene in one sample, whose element is the sum of the corresponding row of the full sampling rate matrix (A_matrix). This vector is generated in one of the rseq output files with the extension ".sampling_rates" and is automatically read into a vector in R through calling the function "read_file" in this "rdiff" package. See the references for a detailed description of sampling-rate row-sum vector.
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~

}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
This function returns the isoform abundance vector theta. See the references for a detailed description of how theta is defined.
}
\references{
%% ~put references to the literature/web site here ~
Jiang, H., Wong, W.H. (2009) Statistical Inferences for Isoform Expression in RNA-Seq, Bioinformatics, 25(8), 1026C1032. 

Salzman, J., Jiang, H., Wong, W. H. (2011) Statistical Modeling of RNA-Seq Data, Statistical Science, 26 (1): 62-83.
}
\author{
%%  ~~who you are~~
Hui Jiang and Yang Shi
}
\note{
%%  ~~further notes~~

}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.
#We give two toy examples here. The first example illustrates how to use this function to calculate the MLE of isoform abundance vector (theta) in one single RNA-seq sample; the second example illustrates how to use this function to calculate the joint isoform abundance vector under model 0.

#The first toy example adopt from the single-end read data for the mouse gene "Rnpep" in section 5.1 of reference [2], where the gene Rnpep has two isoforms. In this example, there are only one condition. Please see the reference for a detailed description of A_matrix, N_vector and W_vector.

A_matrix = matrix( c(4242, 296, 0, 4242, 0, 62)*2762428/1e9, nrow=2, byrow=T)
N_vector = c(216, 10, 0)
W_vector = c(12.53590, 11.88949)

theta_MLE = em_model0(A_matrix, N_vector, W_vector)
#the result is theta_MLE=(15.47, 2.70)

#The second example: we assume there are two conditions, and the two conditions have the same total reads and the same reads count vector for the gene. Therefore, we should expect the optimum model fitting this example is model 0.

A1=matrix( c(4242, 296, 0, 4242, 0, 62)*2762428/1e9, nrow=2, byrow=T) #this is the sampling rate matrix of contion 1
A2=A1 #the sampling rate matrix of condtion 2 is the same as condtion 1
A_matrix_0 = cbind(A1, A2)

N1=c(216, 10, 0) #this is the read count vector of condtion 1
N2=N1 #the read count vector of condtion 2 is the same as condtion 1
N_vector_0 = c(N1, N2)

W1=c(12.53590, 11.88949)#this is the W vector of contion 1
W2=W1#the W vector of condtion 2 is the same as condtion 1
W_vector_0 = W1+W2

theta_0 = em_model0(A_matrix_0, N_vector_0, W_vector_0)
theta_0
#the result is theta_0 = c(15.47, 2.70), which is the joint isoform abundance vector under model 0.

## The function is currently defined as
function (A_matrix, N_vector, W_vector) 
{
    I = length(W_vector)
    if (any(is.na(A_matrix)) && any(is.na(N_vector))) {
        theta = rep(0, times = I)
        return(theta)
    }
    else if (any(is.na(A_matrix)) == F && any(is.na(N_vector)) == 
        F) {
        theta_old = rep(1/I, times = I)
        theta_old_previous = rep(0, times = I)
        num_iter = 0
        while (sum(abs(theta_old - theta_old_previous)) > 1e-06) {
            num_iter = num_iter + 1
            n_i_j_k = matrix(c(t(tcrossprod(theta_old, N_vector))) * 
                c(t(A_matrix))/c(crossprod(theta_old, A_matrix)), 
                nrow = I, byrow = T)
            theta_new = rowSums(n_i_j_k)/W_vector
            theta_old_previous = theta_old
            theta_old = theta_new
        }
        theta = theta_new
        return(theta)
    }
    else {
        print("There is something wrong with either the A_matrix or N_vector. One of them is NULL, but the other is not NULL.")
        return(NA)
    }
  }
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
