\name{isoform_estimation}
\alias{isoform_estimation}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
Estimating and Inferring Differential Isoform Expression for Multi-sample RNA-seq Data
}
\description{
%%  ~~ A concise (1-5 lines) description of what the function does. ~~
This is the "main" function in this package, which tests whether there is differential isoform expression among all the K conditions for each gene, and calculates the maximum likelihood estimation (MLE) of the isoform abundance. Based on the biological significance of differential isoform expression of a gene, this function makes statistical inference on the following three models: model 0, where neither the gene nor its alternative isoforms show differential expression in all the K conditions; model 1, where the gene shows differential expression, but its alternative isoforms do not show differential expression in all the K conditions; model 2, where both the gene and its alternative isoforms show differential expression in all the K conditions (see the reference for a detailed description of how these models are defined). This function calculates the maximum likelihood estimation (MLE) of the isoform abundance and gene abundance under each model, and calculates which model is the optimum model through likelihood ratio test. It also can compute the 95% confidence intervals for the the isoform abundance and the gene abundance under the optimum model, and this option can be specified by the user.
}
\usage{
isoform_estimation(list_K_files, condition_names, CI = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{list_K_files}{
%%     ~~Describe \code{list_K_files} here~~
This is a list containing K elements, where K is the number of conditions in the study. Each element in this list is the returned value from the "read_file" functiuon.
}
  \item{condition_names}{
%%     ~~Describe \code{condition_names} here~~
This is a character vector of length K (K is the number of conditions in the study) given by the user, and each value in the vector should match the name and order of the condition names of the rseq output file read by the "read_file" function, which also should match the element in "list_K_files". If the user does not specify this input value, the program will rank the condition numbers (condition_1, condition_2...) according to the input order by the user. 
}
  \item{CI}{
%%     ~~Describe \code{CI} here~~
This is a logical value of either "TRUE" or "FALSE", which is specified by the user. If it is "TRUE", then the program will calculate the 95% confidence interval for the the isoform abundance of each gene; if it is "FALSE", then the program will not calculate the 95% confidence interval. The default is "FALSE".
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
The returned values is a list, and the length of the list is the number of genes in the study. Each element in the list is the estimation and inference for each gene in the study, which is also a list containing the following two lists:
\item{MLE_under_each_model}
{this is a list containing the maximum likelihood estimation (MLE) of the isoform abundance and gene abundance and the loglikelihood of the MLE under each of the three models(see the "description" above for what the three models are, and the reference for details).It contains the following entries:
\item{Model0_MLE}{contains MLE of the isoform abundance and gene abundance and the loglikelihood of the MLE under model 0.}
\item{Model1_MLE}{contains MLE of the isoform abundance and gene abundance and the loglikelihood of the MLE under model 1.}
\item{Model2_MLE}{contains MLE of the isoform abundance and gene abundance and the loglikelihood of the MLE under model 2.}
}

\item{OPT_model_estimation}
{this is a list containing estimation of the isoform abundance and gene abundance under the optimum model (which is selected by likelihood ratio test). It contains the following entries:
\item{opt_model}{tells you which model is the optimum model selected by the likelihood ratio test, it is either model 0, model 1 or model 2.}
\item{p_value}{the p-values of the likelihood ratio test.}
\item{MLE}{the MLE of the isoform abundance and gene abundance and the loglikelihood of the MLE under the optimum model.}
\item{CI_estimation}{if the user specifies 'CI=TRUE', then this value will be computed. It contains the 95% confidence intervals for the isoform abundance and the gene abundance of each condition under the optimum model.}
}

If there is only one condition/sample in the study (ie. K=1), then the returned value of this function will only be "OPT_model_estimation" and the "opt_model" will be "model_0", also the "p_value" will not come out in this case.
}
\references{
%% ~put references to the literature/web site here ~
Jiang, H., Wong, W.H. (2009) Statistical Inferences for Isoform Expression in RNA-Seq, Bioinformatics, 25(8), 1026C1032. 

Salzman, J., Jiang, H., Wong, W. H. (2011) Statistical Modeling of RNA-Seq Data, Statistical Science, 26 (1): 62-83.
}
\author{
%%  ~~who you are~~
Hui Jiang and Yang Shi
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

#First, read the ".sampling_rates" file of rseq output into a R list through "read_file" function.
file1=read_file("path to the .sampling_rates file of rseq output of sample 1")
file2=read_file("path to the .sampling_rates file of rseq output of sample 2")

#Second, put all the lists in a list
list_two_conditions=list(file1, file2)

#Third, give the list as input to the "isoform_estimation" function. The CI option and condition names can be specified by the user.
result = isoform_estimation(list_two_conditions, condition_names=c("brain", "kidney"), CI=T) #If the user doesn't give the condition_names, the default is "conditon_1" and "condition_2".

## The function is currently defined as
function (list_K_files, condition_names, CI = FALSE) 
{
    K_conditions = length(list_K_files)
    if (missing(condition_names)) {
        print("The user did not specify condition/sample names. The program will rank the condition numbers according to the input order by the user.")
        condition_names = NA
        for (k in 1:K_conditions) {
            condition_names[k] = paste("condition_", k, sep = "")
        }
    }
    if (length(condition_names) != K_conditions) {
        stop("The number of condition/sample names given by the user does not match the number of condtions/samples. \n         Please check the input of condition_names.")
    }
    if (CI == T) {
        library(mvtnorm)
    }
    num_genes = list_K_files[[1]]$number_genes
    merged_file = merge_file(list_K_files, condition_names)
    final_result = list()
    time = NA
    for (num_iter in 1:num_genes) {
        start = Sys.time()
        num_isoform = merged_file[[num_iter]]$I
        gene_name = merged_file[[num_iter]]$gene_name
        isoform_names = merged_file[[num_iter]]$isoform_names
        A_K = merged_file[[num_iter]]$A_matrix
        N_K = merged_file[[num_iter]]$N_vector
        W_K = merged_file[[num_iter]]$W_vector
        condition_zero_read = NULL
        for (k in 1:K_conditions) {
            if (any(is.na(A_K[[k]])) && any(is.na(N_K[[k]]))) {
                condition_zero_read = append(condition_zero_read, 
                  k)
            }
            else if (any(is.na(A_K[[k]])) == F && any(is.na(N_K[[k]])) == 
                F) {
                if (any(is.na(match(0, W_K[[k]]))) == F) {
                  W_K[[k]] = W_K[[k]] + 1e-08
                }
            }
            else {
                print(paste("When calculating gene:", gene_name))
                print(paste("There is an error with either the A_matrix or N_vector under condition", 
                  k, "One of them is NA, but the other is not NA."))
                print("Please check the rseq output files of that gene. The program will continue for the next gene and the estimation for this gene will be marked as NA.")
                final_result[[gene_name]] = logical(0)
                break
            }
        }
        if (is.logical(final_result[[gene_name]])) {
            final_result[[gene_name]] = NA
            next
        }
        if (length(condition_zero_read) > 0) {
            A_matrix_m0 = do.call(cbind, lapply(A_K, na.exclude))
            N_vector_m0 = unlist(lapply(N_K, na.exclude))
            W_vector_m0 = Reduce("+", W_K[-condition_zero_read])
        }
        else {
            A_matrix_m0 = do.call(cbind, A_K)
            N_vector_m0 = unlist(N_K)
            W_vector_m0 = Reduce("+", W_K)
        }
        if (is.logical(A_matrix_m0) && is.logical(N_vector_m0)) {
            opt_model = "model_0"
            theta_MLE = rep(0, times = num_isoform)
            names(theta_MLE) = isoform_names
            loglikelihood_MLE = 0
            gene_MLE = rep(0, times = K_conditions)
            names(gene_MLE) = condition_names
            if (CI == FALSE) {
                final_result[[gene_name]] = list(OPT_model_estimation = list(opt_model = opt_model, 
                  theta_MLE = theta_MLE, loglikelihood_MLE = loglikelihood_MLE, 
                  gene_MLE = gene_MLE))
            }
            else {
                theta_result = matrix(rep(0, times = 3 * num_isoform), 
                  ncol = 3, nrow = num_isoform)
                gene_result = c(0, 0, 0)
                colnames(theta_result) = c("imp_mean", "CI_lower", 
                  "CI_upper")
                names(gene_result) = c("imp_mean", "CI_lower", 
                  "CI_upper")
                CI_estimation = list(theta_CI = theta_result, 
                  gene_CI = gene_result)
                final_result[[gene_name]] = list(OPT_model_estimation = list(opt_model = opt_model, 
                  theta_MLE = theta_MLE, loglikelihood_MLE = loglikelihood_MLE, 
                  gene_MLE = gene_MLE, CI_estimation = CI_estimation))
            }
        }
        else if (is.logical(A_matrix_m0) == F && is.logical(N_vector_m0) == 
            F) {
            theta_MLE_m0 = em_model0(A_matrix_m0, N_vector_m0, 
                W_vector_m0)
            names(theta_MLE_m0) = isoform_names
            loglikelihood_MLE_m0 = log_likelihood_model0(A_matrix_m0, 
                N_vector_m0, W_vector_m0, theta_MLE_m0)
            gene_MLE_model0 = sum(theta_MLE_m0)
            Model0_MLE = list(theta_MLE = theta_MLE_m0, loglikelihood_MLE = loglikelihood_MLE_m0, 
                gene_MLE = gene_MLE_model0)
            if (K_conditions == 1) {
                opt_model = "model_0"
                theta_MLE = Model0_MLE$theta_MLE
                names(theta_MLE) = isoform_names
                loglikelihood_MLE = Model0_MLE$loglikelihood_MLE
                gene_MLE = Model0_MLE$gene_MLE
                names(gene_MLE) = condition_names
                if (CI == FALSE) {
                  final_result[[gene_name]] = list(OPT_model_estimation = list(opt_model = opt_model, 
                    theta_MLE = theta_MLE, loglikelihood_MLE = loglikelihood_MLE, 
                    gene_MLE = gene_MLE))
                }
                else {
                  CI_for_model0 = Calculating_CI_for_model_0(A_matrix_m0, 
                    N_vector_m0, W_vector_m0, theta_MLE_m0, loglikelihood_MLE_m0)
                  rownames(CI_for_model0$theta_CI) = isoform_names
                  final_result[[gene_name]] = list(OPT_model_estimation = list(opt_model = opt_model, 
                    theta_MLE = theta_MLE, loglikelihood_MLE = loglikelihood_MLE, 
                    gene_MLE = gene_MLE, CI_estimation = CI_for_model0))
                }
            }
            if (K_conditions >= 2) {
                model1_em = em_model1(A_K, N_K, W_K)
                theta_MLE_m1 = model1_em$theta
                if (any(is.na(theta_MLE_m1))) {
                  k_error = model1_em$k_error
                  print(paste("Something is wrong with the A_matrix or N_vector of gene:", 
                    gene_name, ", under", condition_names[k_error]))
                  print("Please check the rseq output files of that gene. The program will continue for the next gene and the estimation for this gene will be marked as NA.")
                  final_result[[gene_name]] = NA
                  next
                }
                rownames(theta_MLE_m1) = isoform_names
                ratio_MLE_m1 = model1_em$ratio
                names(ratio_MLE_m1) = condition_names
                theta_MLE_m1_under_each_condition = outer(ratio_MLE_m1, 
                  theta_MLE_m1)
                rownames(theta_MLE_m1_under_each_condition) = condition_names
                colnames(theta_MLE_m1_under_each_condition) = isoform_names
                loglikelihood_MLE_m1 = log_likelihood_model1(A_K, 
                  N_K, W_K, theta_MLE_m1, ratio_MLE_m1)
                gene_MLE_model1 = rowSums(theta_MLE_m1_under_each_condition)
                names(gene_MLE_model1) = condition_names
                Model1_MLE = list(theta_MLE_for_all = theta_MLE_m1, 
                  ratio_MLE = ratio_MLE_m1, theta_MLE_under_each_condition = theta_MLE_m1_under_each_condition, 
                  loglikelihood_MLE = loglikelihood_MLE_m1, gene_MLE = gene_MLE_model1)
                theta_MLE_m2 = matrix(rep(0, times = num_isoform * 
                  K_conditions), nrow = K_conditions)
                loglikelihood_MLE_m2 = rep(0, times = K_conditions)
                for (k in 1:K_conditions) {
                  if (any(is.na(A_K[[k]])) == F && any(is.na(N_K[[k]])) == 
                    F) {
                    theta_MLE_m2[k, ] = em_model0(A_K[[k]], N_K[[k]], 
                      W_K[[k]])
                    loglikelihood_MLE_m2[k] = log_likelihood_model0(A_K[[k]], 
                      N_K[[k]], W_K[[k]], theta_MLE_m2[k, ])
                  }
                }
                rownames(theta_MLE_m2) = condition_names
                colnames(theta_MLE_m2) = isoform_names
                loglikelihood_MLE_m2_sum = sum(loglikelihood_MLE_m2)
                gene_MLE_model2 = rowSums(theta_MLE_m2)
                names(gene_MLE_model2) = condition_names
                Model2_MLE = list(theta_MLE = theta_MLE_m2, loglikelihood_MLE = loglikelihood_MLE_m2_sum, 
                  gene_MLE = gene_MLE_model2)
                MLE_under_each_model = list(Model0_MLE = Model0_MLE, 
                  Model1_MLE = Model1_MLE, Model2_MLE = Model2_MLE)
                LRT_result = likelihood_ratio_test(loglikelihood_MLE_m0, 
                  loglikelihood_MLE_m1, loglikelihood_MLE_m2_sum, 
                  K_conditions, num_isoform)
                opt_model = LRT_result$opt_model
                p_value = LRT_result$p_value
                if (length(p_value) == 2) {
                  names(p_value) = c("model1_vs_model0", "model2_vs_model0")
                }
                if (length(p_value) == 3) {
                  names(p_value) = c("model1_vs_model0", "model2_vs_model0", 
                    "model2_vs_model1")
                }
                if (opt_model == "model_0") {
                  if (CI == TRUE) {
                    CI_for_model0 = Calculating_CI_for_model_0(A_matrix_m0, 
                      N_vector_m0, W_vector_m0, theta_MLE_m0, 
                      loglikelihood_MLE_m0)
                    rownames(CI_for_model0$theta_CI) = isoform_names
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model0_MLE, CI_estimation = CI_for_model0)
                    final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                      OPT_model_estimation = OPT_model_estimation)
                  }
                  else {
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model0_MLE)
                  }
                  final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                    OPT_model_estimation = OPT_model_estimation)
                }
                if (opt_model == "model_1") {
                  if (CI == T) {
                    CI_for_model1 = Calculating_CI_for_model_1(A_K, 
                      N_K, W_K, theta_MLE_m1, ratio_MLE_m1, loglikelihood_MLE_m1)
                    if (any(is.na(CI_for_model1[[1]])) == F) {
                      for (k in 1:K_conditions) {
                        rownames(CI_for_model1[[k]]$theta_CI) = isoform_names
                      }
                      names(CI_for_model1) = condition_names
                    }
                    else {
                      print(paste("The fisher information matrix for gene:", 
                        gene_name, "is ill-conditioned. The 95\% CI for the isoform abundance of that gene is NA."))
                    }
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model1_MLE, CI_estimation = CI_for_model1)
                    final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                      OPT_model_estimation = OPT_model_estimation)
                  }
                  else {
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model1_MLE)
                  }
                  final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                    OPT_model_estimation = OPT_model_estimation)
                }
                if (opt_model == "model_2") {
                  if (CI == T) {
                    CI_for_model2 = Calculating_CI_for_model_2(A_K, 
                      N_K, W_K, theta_MLE_m2, loglikelihood_MLE_m2)
                    for (k in 1:K_conditions) {
                      rownames(CI_for_model2[[k]]$theta_CI) = isoform_names
                    }
                    names(CI_for_model2) = condition_names
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model2_MLE, CI_estimation = CI_for_model2)
                    final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                      OPT_model_estimation = OPT_model_estimation)
                  }
                  else {
                    OPT_model_estimation = list(opt_model = opt_model, 
                      p_value = p_value, MLE = Model2_MLE)
                  }
                  final_result[[gene_name]] = list(MLE_under_each_model = MLE_under_each_model, 
                    OPT_model_estimation = OPT_model_estimation)
                }
            }
        }
        else {
            print(paste("Something is wrong with the A_matrix or N_vector of gene:", 
                gene_name))
            print("Please check the rseq output files of that gene. The program will continue for the next gene and the estimation for this gene will be marked as NA.")
            final_result[[gene_name]] = NA
        }
        end = Sys.time()
        time[num_iter] = end - start
        print(paste("gene", num_iter, "done!"))
    }
    return(list(result = final_result, time = time))
  }
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
